import datetime
import os
import git

number = None

def next_number(path):
    global number
    max_num = 0
    for file in os.listdir(path):
        if "_" in file: file = file.split("_")[0]
        try:
            if int(file) > max_num: max_num = int(file)
        except: pass
    number = max_num + 1
    return number

def new(name = None, verbose=False, **kwargs):
    global number

    if kwargs:
        name = name + "-" + description(**kwargs)

    timestamp = str(datetime.datetime.now()).split(" ")

    try:
        repo = git.Repo(search_parent_directories=True)
        sha  = "_" + repo.head.object.hexsha[:7]
    except: sha = ""

    path = "./archives/" + timestamp[0] + sha

    if not os.path.isdir(path): os.makedirs(path)

    if path is not None: next_number(path)
    else: number = 1

    if name is not None: name = "_" + name
    else: name = ""

    if not os.path.isdir(f"{path}/{number}{name}"): os.makedirs(f"{path}/{number}{name}")

    archive_name = f"{path}/{number}{name}"

    if verbose:
        print(f"Archive created at {archive_name}")

    return archive_name

def description(**kwargs):
    desc = ""
    for key,value in kwargs.items():
        if type(value) in [int, str, float, bool] : desc += f",{key}={value}"
        else: print(f"⚠️ Your archive description contain a non-supported type: {type(value)}")
    return desc[1:]



#!/usr/bin/env python
# coding=utf-8

from datetime import datetime
import os
import platform
from LRFutils import color as C
import sys
import traceback

hide_traceback = False

# Get current time as string
def now(human=False, color=False):
    time = str(datetime.now())
    if human:
        if color:
            return f"{C.Blue}{time[8:10]}/{time[5:7]}/{time[0:4]}{C.NC} at {C.Purple}{time[11:13]}:{time[14:16]}:{time[17:19]}{C.NC}"
        else:
            return f"{time[8:10]}/{time[5:7]}/{time[0:4]} at {time[11:13]}:{time[14:16]}:{time[17:19]}"
    else:
        return time.replace(" ", "_").replace(":", ".")

startTime = now()
filename = f"logs/{startTime}.logs"
if not os.path.isdir("logs"): os.makedirs(f"logs/")

# Getting the current environment
with open(filename, "a") as logsFile:
    logsFile.write(f"ENVIRONMENT: {platform.uname()}\n\n")

# Print message in log file
def logSave(message):
    currentTime = now(human = True, color=False)
    with open(filename, "a", encoding="utf-8") as logsFile:
        logsFile.write(f"{currentTime} | {message}\n")

# Print message in terminal
def logPrint(message):
    currentTime = now(human = True, color=True)
    print(f'{currentTime} | {message}')

# Info-styled messages
def info(message):
    logSave(f"[INFO] {message}")
    message = C.Green + "[INFO] " + C.NC + message
    logPrint(message)
    
# Warning-styled messages
def warn(message):
    message = f"[WARNING] {message}"
    logSave(message)
    message = C.Yellow + message + C.NC
    logPrint(message)

# Error-styled messages
def error(message, etype = None, value = None, tb=None):
    message = f"[ERROR] {message}"
    logSave(message)
    message = C.on_Red + message + C.NC

    if etype is None or value is None or tb is None: tb = traceback.format_exc()
    else: tb = ''.join(traceback.format_exception(etype, value, tb))
    logSave(f"Full traceback below.\n\n{tb}")

    logPrint(message + f"\n -> Look at {C.Green}{filename}{C.NC} for more information.\n")

# Catch unexpected crashes
def myexcepthook(etype, value, tb):
    global hide_traceback
    error(f"🤕 Uh, there is an unexpected error somewhere: {value} ({type})", etype=etype, value=value, tb=tb)
    if not hide_traceback: sys.__excepthook__(etype, value, tb)

sys.excepthook = myexcepthook


from . import color
from time import time
import datetime

class Bar():



    # ________________________________________________________________________________
    # Create a progress bar

    def __init__(self, max:float|int = 1, width:int = 80, prefix:str = "", eta:bool = True,  decimals:int = 0, show_duration:bool = True, average_ETA:int = 10):
        self.max = max
        self.width = width if isinstance(width,int) else 80
        self.prefix = prefix
        self.eta = eta
        self.decimals = decimals
        self.previous_progress = [0]
        self.previous_update = []
        self.lastETA = "-"
        self.start_at = time()
        self.duration = show_duration
        self.average_ETA = average_ETA


    # ________________________________________________________________________________
    # Compute the ETA

    def update_eta(self, progress):

        # Get index of the i-th revious update to average the ETA on i updates
        if len(self.previous_progress) <= self.average_ETA:
            i = 0
        else:
            i = -self.average_ETA

         # Compute progression since the i-th previous update
        progression = progress - self.previous_progress[i]
        todo = self.max - progress

        # If there is no progression or the bar was just created, it update the data but return the same ETA
        if progression == 0 or self.previous_update == []:
            self.previous_update.append(time())
            self.previous_progress.append(progress)
            self.lastETA = "-"
            return self.lastETA

        # If the bar was updated less than one second before,
        # we don't update it again to not slow the program
        if time()-self.previous_update[-1] < 1:
            return self.lastETA

        # Compute the ETA in seconds
        try:
            seconds = todo / progression * (time() - self.previous_update[i])
        except:
            print(len(self.previous_update), i)

        # Update bar data
        self.previous_update.append(time())
        self.previous_progress.append(progress)

        # If the ETA is negative, then we show an undefined ETA
        if seconds < 0:
            self.lastETA = "-"
        # Else we return the number of seconds
        else:
            self.lastETA = str(datetime.timedelta(seconds=seconds)).split(".")[0]
        
        return self.lastETA

    

    # ________________________________________________________________________________
    # Update the progress bar

    def __call__(self, progress: float, prefix=None, stop=False):
        if prefix is None: prefix = self.prefix
        else: prefix = str(prefix)

        progress_normed = progress / self.max
        if progress == self.max : stop = True
        color_bar = color.Yellow if stop and progress_normed != 1 else color.LightGreen 

        if stop: end = "\n"
        else:    end = "\r"

        if type(progress) == float: progress = round(progress, self.decimals)
        percent  = f" {color_bar}{round(progress_normed*100,self.decimals)                                        if self.decimals > 0 else int(progress_normed*100)}%"
        frac     = f" {color.LightRed}{progress}/{self.max}"                                                  if self.max is not None else ''
        eta      = f" {color.NC}eta {color.Blue}{self.update_eta(progress)}"                                  if not stop else ''
        duration = f" {color.Purple}{str(datetime.timedelta(seconds=time() - self.start_at)).split('.')[0]}"  if self.duration else ''

        prefix  = '' if prefix == '' else color.NC + prefix + ' '
        suffix  = f"{percent}{frac}{duration}{eta}"

        barwidth = self.width - len(color.clear(suffix)) - len(color.clear(prefix))
        barwidth = max(barwidth,10)

        currentBar = int(round(min(progress_normed*barwidth,barwidth)))
        minBar     = int(min(progress_normed*barwidth,barwidth))

        if progress_normed == 0:   bar = color.White      + '━' * barwidth
        elif progress_normed == 1: bar = color.LightGreen + '━' * barwidth
        elif currentBar == minBar: bar = color_bar + '━' * currentBar       + color.White      + '╺' + color.White + '━' * (barwidth - currentBar - 1)
        else:                      bar = color_bar + '━' * (currentBar - 1) + color_bar + '╸' + color.White + '━' * (barwidth - currentBar)

        msg = f"{prefix}{bar}{suffix}{color.NC}"
        print(msg, end=end)



    # ________________________________________________________________________________
    # Stop the progress bar

    def stop(self):
        self(self.previous_progress[-1], stop=True)
        self.previous_progress = [0]
        self.lastUpdate = None